class DeviceAPIMixin:
    """API calls for getting device information."""
    def get_hdd_info(self) -> object:
        """
        Gets all HDD and SD card information from Camera
        See examples/response/GetHddInfo.json for example response data.
        :return: response json
        """
        body = [{"cmd": "GetHddInfo", "action": 0, "param": {}}]
        return self._execute_command('GetHddInfo', body)

    def format_hdd(self, hdd_id: [int] = [0]) -> bool:
        """
        Format specified HDD/SD cards with their id's
        :param hdd_id: List of id's specified by the camera with get_hdd_info api. Default is 0 (SD card)
        :return: bool
        """
        body = [{"cmd": "Format", "action": 0, "param": {"HddInfo": {"id": hdd_id}}}]
        r_data = self._execute_command('Format', body)[0]
        if r_data["value"]["rspCode"] == 200:
            return True
        print("Could not format HDD/SD. Camera responded with:", r_data["value"])
        return False



class UserAPIMixin:
    """User-related API calls."""
    def get_online_user(self) -> object:
        """
        Return a list of current logged-in users in json format
        See examples/response/GetOnline.json for example response data.
        :return: response json
        """
        body = [{"cmd": "GetOnline", "action": 1, "param": {}}]
        return self._execute_command('GetOnline', body)

    def get_users(self) -> object:
        """
        Return a list of user accounts from the camera in json format.
        See examples/response/GetUser.json for example response data.
        :return: response json
        """
        body = [{"cmd": "GetUser", "action": 1, "param": {}}]
        return self._execute_command('GetUser', body)

    def add_user(self, username: str, password: str, level: str = "guest") -> bool:
        """
        Add a new user account to the camera
        :param username: The user's username
        :param password: The user's password
        :param level: The privilege level 'guest' or 'admin'. Default is 'guest'
        :return: whether the user was added successfully
        """
        body = [{"cmd": "AddUser", "action": 0,
                 "param": {"User": {"userName": username, "password": password, "level": level}}}]
        r_data = self._execute_command('AddUser', body)[0]
        if r_data["value"]["rspCode"] == 200:
            return True
        print("Could not add user. Camera responded with:", r_data["value"])
        return False

    def modify_user(self, username: str, password: str) -> bool:
        """
        Modify the user's password by specifying their username
        :param username: The user which would want to be modified
        :param password: The new password
        :return: whether the user was modified successfully
        """
        body = [{"cmd": "ModifyUser", "action": 0, "param": {"User": {"userName": username, "password": password}}}]
        r_data = self._execute_command('ModifyUser', body)[0]
        if r_data["value"]["rspCode"] == 200:
            return True
        print("Could not modify user:", username, "\nCamera responded with:", r_data["value"])
        return False

    def delete_user(self, username: str) -> bool:
        """
        Delete a user by specifying their username
        :param username: The user which would want to be deleted
        :return: whether the user was deleted successfully
        """
        body = [{"cmd": "DelUser", "action": 0, "param": {"User": {"userName": username}}}]
        r_data = self._execute_command('DelUser', body)[0]
        if r_data["value"]["rspCode"] == 200:
            return True
        print("Could not delete user:", username, "\nCamera responded with:", r_data["value"])
        return False



class SystemAPIMixin:
    """API for accessing general system information of the camera."""
    def get_general_system(self) -> object:
        """:return: response json"""
        body = [{"cmd": "GetTime", "action": 1, "param": {}}, {"cmd": "GetNorm", "action": 1, "param": {}}]
        return self._execute_command('get_general_system', body, multi=True)

    def get_performance(self) -> object:
        """
        Get a snapshot of the current performance of the camera.
        See examples/response/GetPerformance.json for example response data.
        :return: response json
        """
        body = [{"cmd": "GetPerformance", "action": 0, "param": {}}]
        return self._execute_command('GetPerformance', body)

    def get_information(self) -> object:
        """
        Get the camera information
        See examples/response/GetDevInfo.json for example response data.
        :return: response json
        """
        body = [{"cmd": "GetDevInfo", "action": 0, "param": {}}]
        return self._execute_command('GetDevInfo', body)

    def reboot_camera(self) -> object:
        """
        Reboots the camera
        :return: response json
        """
        body = [{"cmd": "Reboot", "action": 0, "param": {}}]
        return self._execute_command('Reboot', body)



class PtzAPIMixin:
    """
    API for PTZ functions.
    """
    def _send_operation(self, operation, speed, index=None):
        if index is None:
            data = [{"cmd": "PtzCtrl", "action": 0, "param": {"channel": 0, "op": operation, "speed": speed}}]
        else:
            data = [{"cmd": "PtzCtrl", "action": 0, "param": {
                "channel": 0, "op": operation, "speed": speed, "id": index}}]
        return self._execute_command('PtzCtrl', data)

    def _send_noparm_operation(self, operation):
        data = [{"cmd": "PtzCtrl", "action": 0, "param": {"channel": 0, "op": operation}}]
        return self._execute_command('PtzCtrl', data)

    def _send_set_preset(self, operation, enable, preset=1, name='pos1'):
        data = [{"cmd": "SetPtzPreset", "action": 0, "param": {
            "channel": 0, "enable": enable, "id": preset, "name": name}}]
        return self._execute_command('PtzCtrl', data)

    def go_to_preset(self, speed=60, index=1):
        """
        Move the camera to a preset location
        :return: response json
        """
        return self._send_operation('ToPos', speed=speed, index=index)

    def add_preset(self, preset=1, name='pos1'):
        """
        Adds the current camera position to the specified preset.
        :return: response json
        """
        return self._send_set_preset('PtzPreset', enable=1, preset=preset, name=name)

    def remove_preset(self, preset=1, name='pos1'):
        """
        Removes the specified preset
        :return: response json
        """
        return self._send_set_preset('PtzPreset', enable=0, preset=preset, name=name)

    def move_right(self, speed=25):
        """
        Move the camera to the right
        The camera moves self.stop_ptz() is called.
        :return: response json
        """
        return self._send_operation('Right', speed=speed)

    def move_right_up(self, speed=25):
        """
        Move the camera to the right and up
        The camera moves self.stop_ptz() is called.
        :return: response json
        """
        return self._send_operation('RightUp', speed=speed)

    def move_right_down(self, speed=25):
        """
        Move the camera to the right and down
        The camera moves self.stop_ptz() is called.
        :return: response json
        """
        return self._send_operation('RightDown', speed=speed)

    def move_left(self, speed=25):
        """
        Move the camera to the left
        The camera moves self.stop_ptz() is called.
        :return: response json
        """
        return self._send_operation('Left', speed=speed)

    def move_left_up(self, speed=25):
        """
        Move the camera to the left and up
        The camera moves self.stop_ptz() is called.
        :return: response json
        """
        return self._send_operation('LeftUp', speed=speed)

    def move_left_down(self, speed=25):
        """
        Move the camera to the left and down
        The camera moves self.stop_ptz() is called.
        :return: response json
        """
        return self._send_operation('LeftDown', speed=speed)

    def move_up(self, speed=25):
        """
        Move the camera up.
        The camera moves self.stop_ptz() is called.
        :return: response json
        """
        return self._send_operation('Up', speed=speed)

    def move_down(self, speed=25):
        """
        Move the camera down.
        The camera moves self.stop_ptz() is called.
        :return: response json
        """
        return self._send_operation('Down', speed=speed)

    def stop_ptz(self):
        """
        Stops the cameras current action.
        :return: response json
        """
        return self._send_noparm_operation('Stop')

    def auto_movement(self, speed=25):
        """
        Move the camera in a clockwise rotation.
        The camera moves self.stop_ptz() is called.
        :return: response json
        """
        return self._send_operation('Auto', speed=speed)



class DisplayAPIMixin:
    """API calls related to the current image (osd, on screen display)."""

    def get_osd(self) -> object:
        """
        Get OSD information.
        See examples/response/GetOsd.json for example response data.
        :return: response json
        """
        body = [{"cmd": "GetOsd", "action": 1, "param": {"channel": 0}}]
        return self._execute_command('GetOsd', body)

    def get_mask(self) -> object:
        """
        Get the camera mask information.
        See examples/response/GetMask.json for example response data.
        :return: response json
        """
        body = [{"cmd": "GetMask", "action": 1, "param": {"channel": 0}}]
        return self._execute_command('GetMask', body)

    def set_osd(self, bg_color: bool = 0, channel: int = 0, osd_channel_enabled: bool = 0, osd_channel_name: str = "",
                osd_channel_pos: str = "Lower Right", osd_time_enabled: bool = 0,
                osd_time_pos: str = "Lower Right") -> bool:
        """
        Set OSD
        :param bg_color: bool
        :param channel: int channel id
        :param osd_channel_enabled: bool
        :param osd_channel_name: string channel name
        :param osd_channel_pos: string channel position ["Upper Left","Top Center","Upper Right","Lower Left","Bottom Center","Lower Right"]
        :param osd_time_enabled: bool
        :param osd_time_pos: string time position ["Upper Left","Top Center","Upper Right","Lower Left","Bottom Center","Lower Right"]
        :return: whether the action was successful
        """
        body = [{"cmd": "SetOsd", "action": 1, "param": {
            "Osd": {"bgcolor": bg_color, "channel": channel,
                    "osdChannel": {"enable": osd_channel_enabled, "name": osd_channel_name,
                                   "pos": osd_channel_pos},
                    "osdTime": {"enable": osd_time_enabled, "pos": osd_time_pos}
                    }
        }}]
        r_data = self._execute_command('SetOsd', body)[0]
        if r_data["value"]["rspCode"] == 200:
            return True
        print("Could not set OSD. Camera responded with status:", r_data["value"])
        return False



import io

import yaml


class ConfigHandler:
    camera_settings = {}

    @staticmethod
    def load() -> yaml or None:
        try:
            stream = io.open("config.yml", 'r', encoding='utf8')
            data = yaml.safe_load(stream)
            return data
        except Exception as e:
            print("Config Property Error\n", e)
        return None



import io
import random
import string
from urllib import request

from PIL import Image

from RtspClient import RtspClient
from resthandle import Request


class RecordingAPIMixin:
    """API calls for recording/streaming image or video."""
    def get_recording_encoding(self) -> object:
        """
        Get the current camera encoding settings for "Clear" and "Fluent" profiles.
        See examples/response/GetEnc.json for example response data.
        :return: response json
        """
        body = [{"cmd": "GetEnc", "action": 1, "param": {"channel": 0}}]
        return self._execute_command('GetEnc', body)

    def get_recording_advanced(self) -> object:
        """
        Get recording advanced setup data
        See examples/response/GetRec.json for example response data.
        :return: response json
        """
        body = [{"cmd": "GetRec", "action": 1, "param": {"channel": 0}}]
        return self._execute_command('GetRec', body)

    ###########
    # RTSP Stream
    ###########
    def open_video_stream(self, profile: str = "main") -> Image:
        """
        profile is "main" or "sub"
        https://support.reolink.com/hc/en-us/articles/360007010473-How-to-Live-View-Reolink-Cameras-via-VLC-Media-Player
        :param profile:
        :return:
        """
        with RtspClient(ip=self.ip, username=self.username, password=self.password,
                        proxies={"host": "127.0.0.1", "port": 8000}) as rtsp_client:
            rtsp_client.preview()

    def get_snap(self, timeout: int = 3) -> Image or None:
        """
        Gets a "snap" of the current camera video data and returns a Pillow Image or None
        :param timeout: Request timeout to camera in seconds
        :return: Image or None
        """
        randomstr = ''.join(random.choices(string.ascii_uppercase + string.digits, k=10))
        snap = self.url + "?cmd=Snap&channel=0&rs=" \
               + randomstr \
               + "&user=" + self.username \
               + "&password=" + self.password
        try:
            req = request.Request(snap)
            req.set_proxy(Request.proxies, 'http')
            reader = request.urlopen(req, timeout)
            if reader.status == 200:
                b = bytearray(reader.read())
                return Image.open(io.BytesIO(b))
            print("Could not retrieve data from camera successfully. Status:", reader.status)
            return None

        except Exception as e:
            print("Could not get Image data\n", e)
            raise



class NetworkAPIMixin:
    """API calls for network settings."""
    def set_net_port(self, http_port=80, https_port=443, media_port=9000, onvif_port=8000, rtmp_port=1935,
                     rtsp_port=554) -> bool:
        """
        Set network ports
        If nothing is specified, the default values will be used
        :param rtsp_port: int
        :param rtmp_port: int
        :param onvif_port: int
        :param media_port: int
        :param https_port: int
        :type http_port: int
        :return: bool
        """
        body = [{"cmd": "SetNetPort", "action": 0, "param": {"NetPort": {
            "httpPort": http_port,
            "httpsPort": https_port,
            "mediaPort": media_port,
            "onvifPort": onvif_port,
            "rtmpPort": rtmp_port,
            "rtspPort": rtsp_port
        }}}]
        self._execute_command('SetNetPort', body, multi=True)
        print("Successfully Set Network Ports")
        return True

    def set_wifi(self, ssid, password) -> object:
        body = [{"cmd": "SetWifi", "action": 0, "param": {
            "Wifi": {
                "ssid": ssid,
                "password": password
            }}}]
        return self._execute_command('SetWifi', body)

    def get_net_ports(self) -> object:
        """
        Get network ports
        :return: response json
        """
        body = [{"cmd": "GetNetPort", "action": 1, "param": {}},
                {"cmd": "GetUpnp", "action": 0, "param": {}},
                {"cmd": "GetP2p", "action": 0, "param": {}}]
        return self._execute_command('GetNetPort', body, multi=True)

    def get_wifi(self):
        body = [{"cmd": "GetWifi", "action": 1, "param": {}}]
        return self._execute_command('GetWifi', body)

    def scan_wifi(self):
        body = [{"cmd": "ScanWifi", "action": 1, "param": {}}]
        return self._execute_command('ScanWifi', body)



from api import APIHandler


class Camera(APIHandler):

    def __init__(self, ip, username="admin", password="", https=False):
        """
        Initialise the Camera object by passing the ip address.
        The default details {"username":"admin", "password":""} will be used if nothing passed
        :param ip:
        :param username:
        :param password:
        """
        # For when you need to connect to a camera behind a proxy, pass
        # a proxy argument: proxy={"http": "socks5://127.0.0.1:8000"}
        APIHandler.__init__(self, ip, username, password, https=https)

        # Normal call without proxy:
        # APIHandler.__init__(self, ip, username, password)

        self.ip = ip
        self.username = username
        self.password = password
        super().login()



from .recording import RecordingAPIMixin
from .zoom import ZoomAPIMixin
from .device import DeviceAPIMixin
from .display import DisplayAPIMixin
from .network import NetworkAPIMixin
from .system import SystemAPIMixin
from .user import UserAPIMixin
from .ptz import PtzAPIMixin
from resthandle import Request


class APIHandler(SystemAPIMixin,
                 NetworkAPIMixin,
                 UserAPIMixin,
                 DeviceAPIMixin,
                 DisplayAPIMixin,
                 RecordingAPIMixin,
                 ZoomAPIMixin,
                 PtzAPIMixin):
    """
    The APIHandler class is the backend part of the API, the actual API calls
    are implemented in Mixins.
    This handles communication directly with the camera.
    Current camera's tested: RLC-411WS

    All Code will try to follow the PEP 8 standard as described here: https://www.python.org/dev/peps/pep-0008/
    """

    def __init__(self, ip: str, username: str, password: str, https=False, **kwargs):
        """
        Initialise the Camera API Handler (maps api calls into python)
        :param ip:
        :param username:
        :param password:
        :param proxy: Add a proxy dict for requests to consume.
        eg: {"http":"socks5://[username]:[password]@[host]:[port], "https": ...}
        More information on proxies in requests: https://stackoverflow.com/a/15661226/9313679
        """
        scheme = 'https' if https else 'http'
        self.url = f"{scheme}://{ip}/cgi-bin/api.cgi"
        self.ip = ip
        self.token = None
        self.username = username
        self.password = password
        Request.proxies = kwargs.get("proxy")  # Defaults to None if key isn't found

    def login(self) -> bool:
        """
        Get login token
        Must be called first, before any other operation can be performed
        :return: bool
        """
        try:
            body = [{"cmd": "Login", "action": 0,
                     "param": {"User": {"userName": self.username, "password": self.password}}}]
            param = {"cmd": "Login", "token": "null"}
            response = Request.post(self.url, data=body, params=param)
            if response is not None:
                data = response.json()[0]
                code = data["code"]
                if int(code) == 0:
                    self.token = data["value"]["Token"]["name"]
                    print("Login success")
                    return True
                print(self.token)
                return False
            else:
                print("Failed to login\nStatus Code:", response.status_code)
                return False
        except Exception as e:
            print("Error Login\n", e)
            raise

    def logout(self) -> bool:
        """
        Logout of the camera
        :return: bool
        """
        try:
            data = [{"cmd": "Logout", "action": 0}]
            ret = self._execute_command('Logout', data)
            print(ret)
            return True
        except Exception as e:
            print("Error Logout\n", e)
            return False

    def _execute_command(self, command, data, multi=False):
        """
        Send a POST request to the IP camera with given data.
        :param command: name of the command to send
        :param data: object to send to the camera (send as json)
        :param multi: whether the given command name should be added to the
        url parameters of the request. Defaults to False. (Some multi-step
        commands seem to not have a single command name)
        :return: response JSON as python object
        """
        params = {"token": self.token, 'cmd': command}
        if multi:
            del params['cmd']
        try:
            if self.token is None:
                raise ValueError("Login first")
            response = Request.post(self.url, data=data, params=params)
            return response.json()
        except Exception as e:
            print(f"Command {command} failed: {e}")
            raise



import json

import requests


class Request:
    proxies = None

    @staticmethod
    def post(url: str, data, params=None) -> requests.Response or None:
        """
        Post request
        :param params:
        :param url:
        :param data:
        :return:
        """
        try:
            headers = {'content-type': 'application/json'}
            r = requests.post(url, verify=False, params=params, json=data, headers=headers, proxies=Request.proxies)
            # if params is not None:
            #     r = requests.post(url, params=params, json=data, headers=headers, proxies=proxies)
            # else:
            #     r = requests.post(url, json=data)
            if r.status_code == 200:
                return r
            else:
                raise ValueError(f"Http Request had non-200 Status: {r.status_code}", r.status_code)
        except Exception as e:
            print("Post Error\n", e)
            raise

    @staticmethod
    def get(url, params, timeout=1) -> json or None:
        """
        Get request
        :param url:
        :param params:
        :param timeout:
        :return:
        """
        try:
            data = requests.get(url=url, verify=False, params=params, timeout=timeout, proxies=Request.proxies)
            
            return data
        except Exception as e:
            print("Get Error\n", e)
            raise



class ZoomAPIMixin:
    """
    API for zooming and changing focus.
    Note that the API does not allow zooming/focusing by absolute
    values rather that changing focus/zoom for a given time.
    """
    def _start_operation(self, operation, speed):
        data = [{"cmd": "PtzCtrl", "action": 0, "param": {"channel": 0, "op": operation, "speed": speed}}]
        return self._execute_command('PtzCtrl', data)

    def _stop_zooming_or_focusing(self):
        """This command stops any ongoing zooming or focusing actions."""
        data = [{"cmd": "PtzCtrl", "action": 0, "param": {"channel": 0, "op": "Stop"}}]
        return self._execute_command('PtzCtrl', data)

    def start_zooming_in(self, speed=60):
        """
        The camera zooms in until self.stop_zooming() is called.
        :return: response json
        """
        return self._start_operation('ZoomInc', speed=speed)

    def start_zooming_out(self, speed=60):
        """
        The camera zooms out until self.stop_zooming() is called.
        :return: response json
        """
        return self._start_operation('ZoomDec', speed=speed)

    def stop_zooming(self):
        """
        Stop zooming.
        :return: response json
        """
        return self._stop_zooming_or_focusing()

    def start_focusing_in(self, speed=32):
        """
        The camera focuses in until self.stop_focusing() is called.
        :return: response json
        """
        return self._start_operation('FocusInc', speed=speed)

    def start_focusing_out(self, speed=32):
        """
        The camera focuses out until self.stop_focusing() is called.
        :return: response json
        """
        return self._start_operation('FocusDec', speed=speed)

    def stop_focusing(self):
        """
        Stop focusing.
        :return: response json
        """
        return self._stop_zooming_or_focusing()



import socket

import cv2
import numpy
import socks


class RtspClient:

    def __init__(self, ip, username, password, port=554, profile="main", **kwargs):
        """

        :param ip:
        :param username:
        :param password:
        :param port: rtsp port
        :param profile: "main" or "sub"
        :param proxies: {"host": "localhost", "port": 8000}
        """
        self.ip = ip
        self.username = username
        self.password = password
        self.port = port
        self.sockt = None
        self.url = "rtsp://" + self.username + ":" + self.password + "@" + self.ip + ":" + str(
            self.port) + "//h264Preview_01_" + profile
        self.proxy = kwargs.get("proxies")

    def __enter__(self):
        self.sockt = self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.sockt.close()

    def connect(self) -> socket:
        try:
            sockt = socks.socksocket(socket.AF_INET, socket.SOCK_STREAM)
            sockt.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            if self.proxy is not None:
                sockt.set_proxy(socks.SOCKS5, self.proxy["host"], self.proxy["port"])
            sockt.connect((self.ip, self.port))
            return sockt
        except Exception as e:
            print(e)

    def get_frame(self) -> bytearray:
        try:
            self.sockt.send(str.encode(self.url))
            data = b''
            while True:
                try:
                    r = self.sockt.recv(90456)
                    if len(r) == 0:
                        break
                    a = r.find(b'END!')
                    if a != -1:
                        data += r[:a]
                        break
                    data += r
                except Exception as e:
                    print(e)
                    continue
            nparr = numpy.fromstring(data, numpy.uint8)
            frame = cv2.imdecode(nparr, cv2.IMREAD_COLOR)
            return frame
        except Exception as e:
            print(e)

    def preview(self):
        """ Blocking function. Opens OpenCV window to display stream. """
        self.connect()
        win_name = 'RTSP'
        cv2.namedWindow(win_name, cv2.WINDOW_AUTOSIZE)
        cv2.moveWindow(win_name, 20, 20)

        while True:
            cv2.imshow(win_name, self.get_frame())
            # if self._latest is not None:
            #    cv2.imshow(win_name,self._latest)
            if cv2.waitKey(25) & 0xFF == ord('q'):
                break
        cv2.waitKey()
        cv2.destroyAllWindows()
        cv2.waitKey()



#!/usr/bin/python3

import os
import re
import codecs
from setuptools import setup

# Package meta-data.
NAME = 'reolink-api'
DESCRIPTION = 'Reolink Camera API written in Python 3.6'
URL = 'https://github.com/Benehiko/ReolinkCameraAPI'
AUTHOR_EMAIL = ''
AUTHOR = 'Benehiko'
LICENSE = 'GPL-3.0'
INSTALL_REQUIRES = [
    'pillow',
    'pyyaml',
    'requests>=2.18.4',
    'numpy',
    'opencv-python',
    'pysocks'
]


here = os.path.abspath(os.path.dirname(__file__))
# read the contents of your README file
with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


def read(*parts):
    with codecs.open(os.path.join(here, *parts), 'r') as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(name=NAME,
      python_requires='>=3.6.0',
      version=find_version('api', '__init__.py'),
      description=DESCRIPTION,
      long_description=long_description,
      long_description_content_type='text/markdown',
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      url=URL,
      license=LICENSE,
      install_requires=INSTALL_REQUIRES,
      py_modules=[
          'Camera',
          'ConfigHandler',
          'RtspClient',
          'resthandle',
          'api.APIHandler',
          'api.device',
          'api.display',
          'api.network',
          'api.ptz',
          'api.recording',
          'api.system',
          'api.user',
          'api.zoom'
      ]
      )
